
import json
from uuid import uuid4
from PIL import Image, ImageDraw, ImageFont
import requests
from flask import Flask, make_response
import csv
import io

from stats_lib.data_store import InfluxDb
from stats_lib.display.blank_display import BlankDisplay
from stats_lib.display.factory import DisplayFactory
from stats_lib.stats import Stats
from stats_lib.utils import print_debug


influx_db = InfluxDb()

# display = EnergyFlowDisplay()
# display = ZenQuote()
# display = WeatherToday()
# display.draw()
#display._framebuffer.debug_show()


display_factory = DisplayFactory(data_source=influx_db)
app = Flask(__name__)

SCREENS = {}
SHUTDOWN = False

@app.route('/shutdown')
def shutdown():
    """Endpoint to shutdown server"""
    global SHUTDOWN
    SHUTDOWN = True
    return make_response('Screen will turn off in next 5 minutes', 200)

@app.route('/generate')
def generate_screen():
    """Generate screen and return ID"""
    uuid = str(uuid4())

    # If configured to shutdown, return a blank display
    global SHUTDOWN
    if SHUTDOWN:
        display = BlankDisplay()
        SHUTDOWN = False
    else:
        display = display_factory.get_next_display()

    SCREENS[uuid] = display
    response = make_response(json.dumps({"id": uuid}), 200)
    response.headers.set('Content-Type', 'application/json')
    return response

@app.route('/screen/<string:uuid>/<int:line_number>')
def screen_data(uuid, line_number):
    if uuid not in SCREENS:
        return make_response("Screen does not exist", 404)
    response = make_response(SCREENS[uuid].to_byte_array(line_number), 200)
    response.headers.set('Content-Type', 'application/octet-stream')

    # Remove screen after final line is returned
    if line_number == 479:
        print_debug("Deleting screen:", uuid)
        del SCREENS[uuid]
    return response

@app.route('/api/usage-generation-per-day')
def usage_generation_per_day():
    stats = Stats(influx_db).get_energy_usage_generation_per_day()
    fieldnames = ["date", "pv_to_home_kw", "battery_to_home_kw", "pv_to_battery_kw"]
    file = io.StringIO("", "\n")
    writer = csv.DictWriter(file, fieldnames=fieldnames)
    writer.writeheader()

    for date, data in stats.items():
        writer.writerow({
            "date": date,
            "pv_to_home_kw": data.get("pv_to_home_kw"),
            "battery_to_home_kw": data.get("battery_to_home_kw"),
            "pv_to_battery_kw": data.get("pv_to_battery_kw")
        })

    file.seek(0)
    res = make_response(file.read())
    res.headers['Content-Type'] = 'text/csv'
    return res


app.run(debug=True, host="0.0.0.0", threaded=True)
