
import os

from influxdb import InfluxDBClient

from stats_lib.utils import print_debug


class DataStore:

    def __init__(self):
        """Connect to data store"""
        self._connection = self.connect()

    def connect(self):
        """Connect to datastore"""
        raise NotImplementedError

    def store_values(self, data_points):
        """Store datapoints in database."""
        raise NotImplementedError

    def get_latest_value(self, name):
        """Get latest value from datastore"""
        raise NotImplementedError


class DebugPrintDataStore(DataStore):
    """Print out values instead of storing."""

    def connect(self):
        """Do nothing"""
        pass

    def store_values(self, data_points):
        """Print stored values"""
        for data_point in data_points:
            print_debug("Storing datapoint: ", data_point.timestamp, ":", data_point.key, ":", data_point.value)

    def get_latest_value(self, name):
        """Return random value"""
        return 1.232


class InfluxDb(DataStore):

    def connect(self):
        """Connect to influx"""
        self._client = InfluxDBClient(
            host=os.environ.get('INFLUXDB_HOST', 'localhost'),
            port=8086,
            username=os.environ.get('INFLUXDB_USERNAME', None),
            password=os.environ.get('INFLUXDB_PASSWORD', None)
        )

        self._database_name = os.environ.get("INFLUXDB_NAME", "paperdisplay")
        self._retention_policy_name = os.environ.get("INFLUX_RETENTION_POLICY_NAME", "paperdisplay")

        # Create database if it doesn't exist
        if self._database_name not in [db['name'] for db in self._client.get_list_database()]:
            self._client.create_database(self._database_name)

            # Create retention policy for 3 years
            self._client.create_retention_policy(
                self._retention_policy_name,
                "156w", 3,
                database=self._database_name,
                default=True
            )

        # Switch to database
        self._client.switch_database(self._database_name)

    def store_values(self, data_points):
        """Store values in influx"""
        # Convert value to float, if stored as int
        payload = [
            {
                "measurement": data_point.key,
                "tags": {},
                "time": data_point.timestamp.isoformat(),
                "fields": {
                    "value": float(data_point.value) if type(data_point.value) is int else data_point.value
                }
            }
            for data_point in data_points
            if data_point.value is not None
        ]
        print_debug("Storing values in Influx:", payload)
        self._client.write_points(payload)

    def get_latest_value(self, name, default=None):
        """Return latest value from influx"""
        res = self._client.query(
            f"""
            SELECT last(value) FROM "{self._database_name}"."{self._retention_policy_name}"."{name}"
            """
        )
        for point in res.get_points():
            return point.get('last', default)
        return default

    def get_all_values(self, name):
        """Return all values from influx for a given type"""
        res = self._client.query(
            f"""
            SELECT * FROM "{self._database_name}"."{self._retention_policy_name}"."{name}"
            """
        )
        return res.get_points()
