
import datetime
import json
from stats_lib.display.base_display import BaseDisplay


class Event:

    @classmethod
    def from_data(cls, data):
        """Create event object from data"""
        name = data.get("name")
        if not name:
            raise Exception("name is not present")
        date = data.get("date")
        if not date:
            raise Exception("date is not present")

        today = datetime.date.today()

        try:
            end_time = datetime.datetime.strptime(date, "%m-%d").date()
        except:
            print("Date time not valid format MM-DD")

        # Manipulate current year of reminder
        end_time = datetime.date(day=end_time.day, month=end_time.month, year=today.year)
        if end_time < today:
            end_time = datetime.date(day=end_time.day, month=end_time.month, year=today.year + 1)

        # Default to 7 days reminder
        reminder = data.get("reminder", 7)
        if type(reminder) is not int:
            raise Exception("reminder is not valid number")

        start_time = (end_time - datetime.timedelta(days=reminder))

        return cls(name=name, start_time=start_time, end_time=end_time)

    def __init__(self, name, start_time: datetime.date, end_time: datetime.date):
        """Store event data"""
        self._name = name
        self._start_time = start_time
        self._end_time = end_time
        print("adding event", name, start_time, end_time)

    def shown_at_date(self, date):
        """Whether event is shown at date"""
        return date >= self._start_time and date <= self._end_time

    def get_display_text(self):
        """Return text to show on screen"""
        suffix = "th"
        if self._end_time.day in [1, 21, 31]:
            suffix = "st"
        elif self._end_time.day in [2, 22]:
            suffix = "nd"
        elif self._end_time.day in [3, 23]:
            suffix = "rd"

        format = f"%a {self._end_time.day}{suffix} %b"
        return f"{self._end_time.strftime(format)}: {self._name}"


class UpcomingEvents(BaseDisplay):
    """Upcoming events display"""

    EVENTS = None

    SHOW_IN_ROTATION = False

    @classmethod
    def _get_events(cls):
        """Obtain events from events.json"""
        if cls.EVENTS is None:
            try:
                with open("events.json", "r") as fh:
                    events_text = "\n".join(fh.readlines())
                events_data = json.loads(events_text)

                event_objects = []
                for event in events_data:
                    try:
                        event_objects.append(Event.from_data(event))
                    except Exception as exc:
                        print("failed to read event:", event)
                        raise
                cls.EVENTS = event_objects

            except Exception as exc:
                print("Failed to read events.json:", str(exc))

        return cls.EVENTS or []

    def draw(self):
        """Draw upcoming events"""
        super().draw()
        today = datetime.date.today()

        self._framebuffer.text("Upcoming Events", (200, 100), font_size=40)
        current_screen_itx = 160
        events_shown = False

        for event in self._get_events():
            if event.shown_at_date(today):
                self._framebuffer.text(event.get_display_text(), (20, current_screen_itx))
                current_screen_itx += 40
                events_shown = True

        if not events_shown:
            self._framebuffer.text("There are no upcoming events...", (20, current_screen_itx))



