
from PIL import Image, ImageDraw, ImageFont


class FrameBuffer:

    WIDTH = 800
    HEIGHT = 480

    BLACK = (0, 0, 0)
    WHITE = (255, 255, 255)
    RED = (255, 0, 0)

    def __init__(self):
        """Initialise image"""
        self._image = Image.new("RGB", (self.WIDTH, self.HEIGHT), self.WHITE)
        self._draw = ImageDraw.Draw(self._image)
        self._fonts = {}
        self._image_data = None

    def debug_show(self):
        """Show image"""
        self._image.show()

    @property
    def image_data(self):
        """Return cached image data"""
        if self._image_data is None:
            self._image_data = list(self._image.getdata())
        return self._image_data

    def to_byte_array(self, line_number):
        """Convert image to byte array"""
        image_data = self.image_data[(line_number * self.WIDTH):((line_number + 1) * self.WIDTH)]
        # Convert each pixel to 2 bits -
        # - 00 - white
        # - 01 - black
        # - 10 - red
        # storing 4 pixels per byte
        output_data = b''
        current_byte = 0
        current_pixel = 0
        for itx, pixel_value in enumerate(image_data):
            if pixel_value == self.RED:
                current_byte |= (1 << ((current_pixel * 2) + 1))
            elif pixel_value == self.WHITE:
                # Do not set any values for white
                pass
            else:
                # Treat all other colours as black, if the pillow library has
                # caused any merging of colours etc.
                current_byte |= (1 << (current_pixel * 2))

            # # DEBUG
            # if itx > 4:
            #     break
            # else:
            #     print(current_byte)

            # If last pixel for the byte, push byte to image data array
            # and reset current pixel/byte.
            # Also do this for the final pixel, so the byte is pushed to the array
            if current_pixel == 3 or itx == len(image_data):
                output_data += current_byte.to_bytes(length=1, byteorder='little')
                current_byte = 0
                current_pixel = 0
            else:
                current_pixel += 1

        return output_data

    def text(self, text, location, font_size=30, split_multiline=False, max_width=None, color=BLACK):
        """Draw text"""
        if str(font_size) not in self._fonts:
            self._fonts[str(font_size)] = ImageFont.truetype(f"./stats_lib/fonts/Bookerly-Regular.ttf", font_size)
        font = self._fonts[str(font_size)]

        if split_multiline:
            # Default max height and width to edge of screen,
            # with small buffer
            if not max_width:
                max_width = (self.WIDTH - location[0]) - 40

            words = text.split()

            current_line = ""
            current_y = location[1]
            for word_itx, word in enumerate(words):
                new_line = current_line
                if new_line:
                    new_line += " "
                new_line += word

                line_size_w, line_size_h = self._draw.textsize(text=new_line, font=font)

                # If length of line exceeds the max width,
                # draw the previous line
                if line_size_w > max_width:
                    self._draw.text((location[0], current_y), current_line, fill=color, font=font)
                    current_y += line_size_h
                    current_line = word

                # Otherwise, if on last line, draw line
                elif word_itx == (len(words) - 1):
                    self._draw.text((location[0], current_y), new_line, fill=color, font=font)
                    current_y += line_size_h
                else:
                    # Otherwise, update current line to new line, containing
                    # new word
                    current_line = new_line

            return current_y

        self._draw.text(location, text, fill=color, font=font)
