"""
API
Type: Solar Panel Data
Provider: GivEnergy
Provider Link: https://www.givenergy.cloud
Status: In Development
"""

from enum import Enum
import datetime
import os

import requests

from stats_lib.utils import print_debug
from stats_lib.data_point import DataPoint


class GivEnergy:
    """Interface to interact with givenergy."""

    class EnergyFlowTypes(Enum):
        # The amount of energy generated by the solar panels to be used immediately for local consumption
        PV_TO_HOME = "0"
        # The amount of energy generated by the solar panels that has been used to charge the battery
        PV_TO_BATTERY = "1"
        # The amount of energy generated by the solar panels that has been exported to the grid
        PV_TO_GRID = "2"
        # The amount of energy imported from the grid that has been used immediately for local consumption
        GRID_TO_HOME = "3"
        # The amount of energy imported from the grid that has been used to charge the battery
        GRID_TO_BATTERY = "4"
        # The amount of energy discharged by the battery that has been used for local consumption
        BATTERY_TO_HOME = "5"
        # The amount of energy discharged by the battery that has been exported to the grid
        BATTERY_TO_GRID = "6"


    class GroupingIds(Enum):
        """List of group mapping IDs"""
        HALF_HOURLY = 0
        DAILY = 1
        MONTHLY = 2
        YEARLY = 3
        TOTAL = 4


    API_KEY = os.environ.get('GIVENERGY_API_KEY')

    def __init__(self):
        """Setup information about inverters"""
        self._inverter_serial = self._get_inverter_serial()

    def _get_inverter_serial(self):
        """Get interter serial number."""
        communication_devices = self._make_api_call(
            "communication-device"
        )

        if len(communication_devices["data"]) == 0:
            raise Exception("Found no inverters for GivEnergy")

        if len(communication_devices["data"]) > 1:
            raise Exception("Found more than 1 GivEnergy inverters")

        return communication_devices["data"][0].get("inverter", {}).get("serial")

    def _make_api_call(self, endpoint, params=None, method=requests.get):
        """Make API call to octopus API"""
        if params is None:
            params = {}
        url = f"https://api.givenergy.cloud/v1/{endpoint}"

        print_debug("Calling", url, "with", params)

        kwargs = {
            'timeout': 30
        }
        if method is requests.get:
            kwargs['params'] = params
        elif method is requests.post:
            kwargs['json'] = params
        else:
            raise Exception('Unsupported method')

        try:
            res = method(
                url,
                headers={
                    "Authorization": f"Bearer {self.API_KEY}",
                    "Accept": "application/json",
                    "Content-Type": "application/json"
                },
                **kwargs
            )
        except requests.exceptions.ConnectTimeout:
            return None

        if res.status_code == 200:
            json_response = res.json()
            print_debug("Got response", json_response)
            return json_response
        else:
            print('Error response from GivEnergy: ', res.status_code, res.content)
            return None

    def _convert_datetime_to_string(self, datetime):
        """Convert datetime to string"""
        return datetime.strftime("%Y-%m-%dT%H:%M:%SZ")

    def _convert_date_to_string(self, datetime):
        """Convert datetime object to date string"""
        return datetime.strftime("%Y-%m-%d")

    def get_data_points(self):
        """Obtain data points"""
        date = datetime.datetime.now().strftime('%Y-%m-%d')
        # Since data is filtered by day, obtain latest page number from initial page
        first_page = self._make_api_call(f"inverter/{self._inverter_serial}/data-points/{date}", params={"pageSize": 50})
        last_page = first_page.get("meta", {}).get("last_page", None)
        if not last_page:
            print("Invalid data from GivEnergy data_points initial page")
            return []

        page_list = [last_page - 1, last_page] if last_page > 1 else [last_page]

        data_points = []
        for page_number in page_list:
            page_data = self._make_api_call(f"inverter/{self._inverter_serial}/data-points/{date}", params={"pageSize": 50, "page": page_number})

            for data in page_data["data"]:
                data_dt = datetime.datetime.strptime(data.get("time"), "%Y-%m-%dT%H:%M:%SZ")

                power = data.get("power", {})

                def create_data_point(key, val):
                    data_points.append(DataPoint(key=key, value=val, timestamp=data_dt))

                create_data_point("solar_power_w", power.get("solar", {}).get("power", None))
                create_data_point("grid_frequency", power.get("grid", {}).get("frequency", None))
                create_data_point("grid_power_w", power.get("grid", {}).get("power", None))
                create_data_point("grid_voltage", power.get("grid", {}).get("voltage", None))
                create_data_point("grid_current", power.get("grid", {}).get("current", None))
                create_data_point("battery_power_w", power.get("battery", {}).get("power", None))
                create_data_point("battery_charge_perc", power.get("battery", {}).get("percent", None))
                create_data_point("battery_temperature", power.get("battery", {}).get("temperature", None))
                create_data_point("usage_consumption_w", power.get("consumption", {}).get("power", None))
                create_data_point("inverter_power_w", power.get("inverter", {}).get("power", None))
                create_data_point("inverter_temperature", power.get("inverter", {}).get("temperature", None))
                create_data_point("inverter_eps_power", power.get("inverter", {}).get("eps_power", None))
                create_data_point("inverter_voltage", power.get("inverter", {}).get("output_voltage", None))
                create_data_point("inverter_frequency", power.get("inverter", {}).get("output_frequency", None))

        return data_points

    def get_energy_flow(self):
        """Get energy flow data"""
        # Make request to obtain data for entire day
        start_time = datetime.date.today()
        end_time = start_time + datetime.timedelta(days=1)

        data = self._make_api_call(
            f"inverter/{self._inverter_serial}/energy-flows",
            {"grouping": self.GroupingIds.HALF_HOURLY.value,
             "start_time": self._convert_date_to_string(start_time),
             "end_time": self._convert_date_to_string(end_time)},
            method=requests.post
        )

        # Iterate over resulting data, adding all data points to get current
        # data for today
        current_data = {}
        data_points = []
        now = datetime.datetime.now()
        for data_point_itx in data.get("data", {}):
            data_point = data["data"][data_point_itx]["data"]
            start_dt = datetime.datetime.strptime(
                data["data"][data_point_itx]["start_time"],
                "%Y-%m-%d %H:%M"
            )
            end_dt = datetime.datetime.strptime(
                data["data"][data_point_itx]["end_time"],
                "%Y-%m-%d %H:%M"
            )

            for data_type in data_point:
                data_type_enum = self.EnergyFlowTypes(data_type)

                this_data_point_has_value = False

                # Add current data data point
                current_data_points = []
                if start_dt <= now:
                    current_data_points.append(DataPoint(
                        key=f"{data_type_enum.name.lower()}_now_kw",
                        value=float(data_point[data_type]),
                        timestamp=end_dt
                    ))
                    if float(data_point[data_type]):
                        this_data_point_has_value = True

                    # Only store data points if at least one has a non-zero number
                    if this_data_point_has_value:
                        for data_point_itx in current_data_points:
                            data_points.append(data_point_itx)

                # Add data point to daily sums
                if data_type_enum not in current_data:
                    current_data[data_type_enum] = 0.0
                current_data[data_type_enum] += float(data_point[data_type])

        for data_type in current_data:
            data_points.append(DataPoint(key=f"{data_type.name.lower()}_kw", value=current_data[data_type]))

        return data_points
