"""
API
Type: Energy Supplier
Provider: Octopus Energy
Provider Link: https://octopus.energy
Status: In Development
"""

import datetime
import json
import os
from re import L

import jwt
import requests

from stats_lib.utils import print_debug


class OctopusEnergySupplyDetails:

    def __init__(self):
        """Store details about octopus energy electricity/gas supply costs"""
        self._unit_cost = None
        self._meter_mpan = None
        self._tariff_code = None
        self._serial_number = None
        self._id = None

    def set_meter_mpan(self, meter_mpan):
        """Set meter mpan"""
        if self._meter_mpan:
            raise Exception('meter MPAN has already been set')
        self._meter_mpan = meter_mpan

    def set_tariff_code(self, tariff_code):
        """Set meter mpan"""
        if self._tariff_code:
            raise Exception('Tariff code has already been set')
        self._tariff_code = tariff_code

    def update_unit_cost(self, unit_cost):
        """Update unit cost"""
        self._unit_cost = unit_cost

    def set_serial_number(self, serial_number):
        """Set serial number"""
        if self._serial_number:
            raise Exception("Serial number has already been set")
        self._serial_number = serial_number

    @property
    def serial_number(self):
        """Return serial number"""
        if not self._serial_number:
            raise Exception("Obtaining serial number before it has been set")
        return self._serial_number

    def set_id(self, id):
        """Set ID"""
        if self._id:
            raise Exception("ID has already been set")
        self._id = id

    @property
    def id(self):
        """Return ID"""
        if not self._id:
            raise Exception("Obtaining ID before it has been set")
        return self._id

    @property
    def unit_cost(self):
        """Return unit cost"""
        if self._unit_cost is None:
            raise Exception('Obtaining unit cost before it has been set')
        return self._unit_cost

    @property
    def meter_mpan(self):
        """Return meter mpan"""
        if self._meter_mpan is None:
            raise Exception('Obtaining meter MPAN before it has been set')

        return self._meter_mpan

    @property
    def tariff_code(self):
        """Return tariff code"""
        if self._tariff_code is None:
            raise Exception('Obtaining tariff code before it has been set')

        return self._tariff_code


class OctopusEnergy:
    """Interface to obtain data from octopus energy"""

    API_KEY = os.environ.get('OCTOPUS_API_KEY')

    def __init__(self):
        """Obtain base details about supply and meters"""
        self._gas = OctopusEnergySupplyDetails()
        self._electricity = OctopusEnergySupplyDetails()
        self._product_code = self._get_product_code()
        self._get_tariff_codes()
        self._get_mpan_codes()
        self.update_tariff_unit_costs()

    def _get_product_code(self):
        """Obtain products to obtain product code"""
        products = self._make_api_call('v1/products')

        if len(products['results'] != 1):
            raise Exception('OctopusEnergy returned none or multiple products')

        return products['results'][0]['code']

    def _get_tariff_codes(self):
        """Obtain tarrif codes for electicity and gas"""
        product_details = self._make_api_call(
            f"/v1/products/{self._product_code}/"
        )

        def obtain_tariff_code_from_fuel_details(fuel_details):
            """Obtain tariff code from fuel tariffs"""
            # Iterate over random _A, _B etc. keys, looking for first with valid
            # content
            for itx_a in fuel_details:
                if fuel_details[itx_a]:
                    # Iterate over payment types: direct_debit_monthly, direct_debit_quarterly etc.
                    for payment_type in fuel_details[itx_a]:
                        if fuel_code := fuel_details[itx_a][payment_type].get('code'):
                            return fuel_code

            return None

        if electricity_tariff_data := product_details.get('single_register_electricity_tariffs'):
            self._electricity.set_tariff_code(obtain_tariff_code_from_fuel_details(electricity_tariff_data))
        else:
            print("No electricity tariffs found")

        if gas_tariff_data := product_details.get('single_register_gas_tariffs'):
            self._gas.set_tariff_code(obtain_tariff_code_from_fuel_details(gas_tariff_data))
        else:
            print("No gas tariffs found")

    def _get_mpan_codes(self):
        """Obtain MPAN codes for gas/electricity meters"""
        account = self._make_api_call(
            "/v1/accounts/"
        )

        def get_mpan_from_meter_points(meter_point_details):
            """Return mpan from list of meter points"""
            for meter_details in meter_point_details:
                if mpan := meter_details.get('mpan'):
                    return mpan

        # Obtain electricity mpan
        self._gas.set_meter_mpan(get_mpan_from_meter_points(account.get("electricity_meter_points") or []))

        # Obtain gas mpan
        self._gas.set_meter_mpan(get_mpan_from_meter_points(account.get("gas_meter_points") or []))

    def _make_api_call(self, endpoint, params=None):
        """Make API call to octopus API"""
        if params is None:
            params = {}

        url = f"https://api.octopus.energy/{endpoint}"

        print_debug("Calling", url, "with", params)

        res = requests.get(
            url,
            # Supply API key as username and empty password
            auth=(self.API_KEY, ''),
            params=params
        )

        if res.status_code == 200:
            json_response = res.json()
            print_debug("Got response", json_response)
            return json_response
        else:
            print('Error response from OctopusEnergy: ', res.status_code, res.content)
            return None


class SmartDevice:

    def __init__(self, type, serial_number, id):
        """Store member variables"""
        self._type = type
        self._serial_number = serial_number
        self._id = id


class OctopusEnergyGraphQL:
    """Interface to obtain data from octopus energy via GraphQL interface"""

    def __init__(self, email, password):
        """Obtain base details about supply and meters"""
        self._gas = OctopusEnergySupplyDetails()
        self._electricity = OctopusEnergySupplyDetails()
        self._email = email
        self._password = password
        self._auth_token = None
        self._auth_expiration = None
        self._smart_devices = []
        self._account_number = self.get_account_number()
        self.get_account_details()

    def _authenticate_with_email_password(self):
        """Authenticate with email and password to Kraken"""
        res = self._make_api_call(
            "krakenTokenAuthentication",
            """
            mutation krakenTokenAuthentication($email: String!, $password: String!) {
                obtainKrakenToken(input: {email: $email, password: $password}) {
                    token
                }
            }
            """,
            {"email": self._email, "password": self._password},
            authenticated_required=False)
        return res.get("data", {}).get("obtainKrakenToken", {}).get("token")

    def _get_authorization_token(self):
        """Obtain active authorization token"""
        # If auth token or expiration has not been set,
        # or token has expired or will expire in 1 minute, renew
        if ((not self._auth_token) or (not self._auth_expiration) or
                self._auth_expiration < (datetime.datetime.now() - datetime.timedelta(minutes=1))):
            self._auth_token = self._authenticate_with_email_password()

            header = jwt.get_unverified_header(self._auth_token)
            decoded = jwt.decode(
                self._auth_token,
                options={"verify_signature": False},
                algorithms=[header["alg"]]
            )
            self._auth_expiration = datetime.datetime.fromtimestamp(decoded.get("exp"))

        return self._auth_token

    def get_account_number(self):
        """Obtain account number"""
        res = self._make_api_call(
            "getLoggedInUser",
            """
            query getLoggedInUser { 
                viewer {
                    accounts {
                        number
                        __typename
                    }
                    __typename
                }
            }
            """
        )
        for account in res.get("data", {}).get("viewer", {}).get("accounts", []):
            if account_number := account.get("number"):
                return account_number

    def get_account_details(self):
        """Get account meters and smart devices"""
        res = self._make_api_call(
            "getAccountPropertiesMeters",
            """
            query getAccountPropertiesMeters($accountNumber: String!) {
                account(accountNumber: $accountNumber) {
                    properties {
                        id
                        smartDeviceNetworks {
                            id
                            smartDevices {
                                id
                                serialNumber
                                type
                            }
                        }
                        electricityMeterPoints {
                            __typename
                            mpan
                            status
                            meters(includeInactive: false) {
                                nodeId
                                id
                                serialNumber
                                consumptionUnits
                                hasAndAllowsHhReadings
                                importMeter {
                                    id
                                    __typename
                                }
                                __typename
                            }
                        }
                        gasMeterPoints {
                            __typename
                            mprn
                            status
                            meters(includeInactive: false) {
                                nodeId
                                id
                                serialNumber
                                consumptionUnits
                                hasAndAllowsHhReadings
                                __typename
                            }
                        }
                        __typename
                    }
                    __typename 
                }
            }
            """,
            {"accountNumber": self._account_number}
        )
        for property in res.get("data", {}).get("account", {}).get("properties", []):
            self._property_id = property.get("id")

            # Set electricity/gas MPAN and serial numbers
            for electricity_meter_point in property.get("electricityMeterPoints"):
                if electricity_meter_point.get("status", "") == "ON_SUPPLY":
                    self._electricity.set_meter_mpan(electricity_meter_point.get("mpan"))
                    for meter in electricity_meter_point.get("meters", []):
                        self._electricity.set_serial_number(meter.get("serialNumber"))
                        self._electricity.set_id(meter.get("id"))

            for gas_meter_point in property.get("gasMeterPoints"):
                if gas_meter_point.get("status", "") == "ON_SUPPLY":
                    self._gas.set_meter_mpan(gas_meter_point.get("mpan"))
                    for meter in gas_meter_point.get("meters", []):
                        self._gas.set_serial_number(meter.get("serialNumber"))
                        self._gas.set_id(meter.get("id"))

            # Get details for all smart devices
            for smart_device in property.get("smartDeviceNetworks"):
                self._smart_devices.append(SmartDevice(
                    id=smart_device.get("id"),
                    serial_number=smart_device.get("serial_number"),
                    type=smart_device.get("type")
                ))

    def _make_api_call(self, operation_name, query, variables=None, authenticated_required=True):
        """Make API call to octopus API"""
        if variables is None:
            variables = {}

        url = f"https://api.octopus.energy/v1/graphql/"

        print_debug("Calling", url, "with", operation_name, ", query:", query, "variables:", variables)

        headers = {
            "Content-Type": "application/json"
        }

        if authenticated_required:
            headers["Authorization"] = self._get_authorization_token()

        try:
            res = requests.post(
                url,
                # Supply API key as username and empty password
                data=json.dumps({
                    "operationName": operation_name,
                    "query": query,
                    "variables": variables
                }),
                headers=headers,
                timeout=30
            )
        except requests.exceptions.ConnectTimeout:
            return None

        if res.status_code == 200:
            json_response = res.json()
            print_debug("Got response", json_response)
            return json_response
        else:
            print(f"Error response from {self.__class__.__name__}: ", res.status_code, res.content)
            return None

    def getSmartElectricityMeterReadings(self):
        """Get electricity meter readings"""
        for device in self._smart_devices:
            self._make_api_call(
                "getSmartMeterTelemetry",
                """
                query getSmartMeterTelemetry($deviceId: String!, $start: DateTime!, $end: DateTime!, $grouping: TelemetryGrouping!) {
                    smartMeterTelemetry(deviceId: $deviceId, start: $start, end: $end, grouping: $grouping) {
                        readAt
                        consumption
                        demand
                        consumptionDelta
                        costDelta
                    }
                }
                """,
                {
                    "deviceId": device._id,
                    "start": datetime.datetime.now().isoformat(),
                    "end": (datetime.datetime.now() - datetime.timedelta(hours=1)).isoformat(),
                    "grouping": "TEN_SECONDS"
                }
            )

    def getElectricityMeterReadings(self):
        """Get electricity meter readings"""
        self._make_api_call(
            "getElectricityMeterReadings",
            """
            query getElectricityMeterReadings($accountNumber: String!, $meterId: String!) {
                electricityMeterReadings(accountNumber: $accountNumber, meterId: $meterId, first: 1) {
                    edges {
                        node {
                            id
                            readAt
                            readingSource
                            registers {
                                name
                                value
                                digits
                            }
                            source
                        }
                    }
                }
            }
            """,
            {
                "accountNumber": self._account_number,
                "meterId": self._electricity.id
            }
        )
