"""
API
Type: Solar Panel Data
Provider: SolarEdge
Provider Link: https://www.solaredge.com/
Status: In Development
"""

import datetime
import os

import requests

from stats_lib.data_point import DataPoint
from stats_lib.utils import print_debug, DEBUG


class SolarEdge:
    """Interact with SolarEdge API"""

    API_KEY = os.environ.get('SOLAR_EDGE_API_KEY')
    # Site ID - Can be obtained from https://monitoringapi.solaredge.com/sites/list?api_key=API_KEY
    SITE_ID = os.environ.get('SOLAR_EDGE_SITE_ID')

    def __init__(self):
        """Obtain core details about inverters"""
        self._inverter_serial = self.get_inverter_serial()

    def _datetime_to_api_value(self, dt):
        """Convert datetime to API value"""
        return dt.strftime("%Y-%m-%d %H:%M:%S")

    def _make_api_call(self, endpoint, params=None, base_uri="site"):
        """Make an API call to solar edge API"""
        if params is None:
            params = {}

        get_param_string = '&'.join([f'{key}={params[key].replace(" ", "%20")}' for key in params])
        url = f"https://monitoringapi.solaredge.com/{base_uri}/{self.SITE_ID}/{endpoint}?api_key={self.API_KEY}&{get_param_string}"

        print_debug("Calling", url, "with", params)

        try:
            res = requests.get(
                url,
                headers={'Accept': 'application/json'},
                timeout=30
            )
        except requests.exceptions.ConnectTimeout:
            return None

        if res.status_code == 200:
            json_response = res.json()
            print_debug("Got response", json_response)
            return json_response
        else:
            print('Error response from SolarEdge: ', res.status_code, res.content)
            return None

    def get_site_power_details(self, offset, time_back):
        """Get detailed site power usage - consumption, export and import"""
        to_date = datetime.datetime.now() - offset
        from_date = to_date - time_back

        # Return cached value
        if DEBUG:
            return {'powerDetails': {'timeUnit': 'QUARTER_OF_AN_HOUR', 'unit': 'W', 'meters': [{'type': 'Production', 'values': [{'date': '2022-12-18 14:45:00', 'value': 4.1847005}]}]}}

        return self._make_api_call(
            "powerDetails",
            {"meters": "PRODUCTION,CONSUMPTION,SELFCONSUMPTION,FEEDIN,PURCHASED",
             "startTime": self._datetime_to_api_value(from_date),
             "endTime": self._datetime_to_api_value(to_date)}
        )

    def get_power_flow(self):
        """Get power flow details"""
        return self._make_api_call(
            "currentPowerFlow",
            {}
        )

    def get_inverter_serial(self):
        """Get inverter serial number"""
        inventory = self._make_api_call(
            "inventory"
        )

        inverters = inventory.get("Inventory").get("inverters")

        if not len(inverters):
            raise Exception(f"Expected 1 inverter from SolarEdge, but got {len(inverters)}")

        print_debug("SolarEdge inverter details", inverters[0])

        return inverters[0].get("SN")

    def get_inverter_technical_data(self, time_back):
        """Return inverter technical data"""
        to_date = datetime.datetime.now()
        from_date = to_date - time_back

        return self._make_api_call(
            f"{self._inverter_serial}/data",
            {
                "startTime": self._datetime_to_api_value(from_date),
                "endTime": self._datetime_to_api_value(to_date)
            },
            base_uri="equipment"
        )

    def get_environment_benefits(self):
        """Get environment benefits data"""
        data = self._make_api_call(
            "envBenefits",
            params={"systemUnits": "Metrics"}
        )
        env_benefits = data.get("envBenefits", {})
        gas_emissions_saved = env_benefits.get("gasEmissionSaved")
        return [
            DataPoint("env_benefits_gas_emission_saved_co2_kg", gas_emissions_saved.get("co2", 0)),
            DataPoint("env_benefits_gas_emission_saved_so2_kg", gas_emissions_saved.get("so2", 0)),
            DataPoint("env_benefits_gas_emission_saved_nox_kg", gas_emissions_saved.get("nox", 0)),
            DataPoint("env_benefits_trees_planted", env_benefits.get("treesPlanted", 0)),
            DataPoint("env_benefits_lightbulbs", env_benefits.get("lightBulbs", 0)),
        ]

    def get_storage_data(self):
        """Get storage data"""
        to_date = datetime.datetime.now()
        from_date = to_date - datetime.timedelta(hours=1)
        data = self._make_api_call(
            "storageData.json",
            params={
                "startTime": self._datetime_to_api_value(from_date),
                "endTime": self._datetime_to_api_value(to_date)
            }
        )
        return data
