"""
API
Type: Weather data
Provider: Weather API
Provider Link: https://www.weatherapi.com
Status: In Development
"""


from enum import Enum
import os

import requests

from stats_lib.utils import print_debug
from stats_lib.data_point import DataPoint


class WeatherAPI:

    API_KEY = os.environ.get("WEATHER_API_API_KEY")
    LAT = os.environ.get("LOCATION_LAT")
    LON = os.environ.get("LOCATION_LON")

    def _make_api_call(self, endpoint, params=None):
        """Make API call"""
        if not self.API_KEY:
            print("No API key present for OpenWeatherMap")
            return None

        if params is None:
            params = {}

        url = f"https://api.weatherapi.com/v1/{endpoint}"

        params['key'] = self.API_KEY

        print_debug("Calling", url, "with", params)

        try:
            res = requests.get(
                url,
                headers={
                    "Accept": "application/json",
                    "Content-Type": "application/json"
                },
                params=params,
                timeout=30
            )
        except requests.exceptions.ConnectTimeout:
            return None

        if res.status_code == 200:
            json_response = res.json()
            print_debug("Got response", json_response)
            return json_response
        else:
            print(f"Error response from {self.__class__.__name__}: ", res.status_code, res.content)
            return None

    def get_current_weather(self):
        """Get current weather conditions"""
        lat = self.LAT
        lon = self.LON
        if not lat or not lon:
            return

        data = self._make_api_call(
            "forecast.json",
            {
                "q": f"{lat},{lon}",
                "days": "1",
                "aqi": "no",
                "alerts": "no"
            }
        )
        current = data.get("current", {})
        return [
            DataPoint("weather_api_location", data.get("location", {}).get("name")),
            DataPoint("weather_api_temp_c", current.get("temp_c")),
            DataPoint("weather_api_is_day", current.get("is_day")),
            DataPoint("weather_api_weather_condition_name", current.get("condition", {}).get("text", "")),
            DataPoint("weather_api_weather_condition_code", current.get("condition", {}).get("code")),
            DataPoint("weather_api_wind_mph", current.get("wind_mph")),
            DataPoint("weather_api_wind_degree", current.get("wind_degree")),
            DataPoint("weather_api_wind_dir", current.get("wind_dir")),
            DataPoint("weather_api_pressure_mb", current.get("pressure_mb")),
            DataPoint("weather_api_pressure_in", current.get("pressure_in")),
            DataPoint("weather_api_precip_in", current.get("precip_in")),
            DataPoint("weather_api_humidity", current.get("humidity")),
            DataPoint("weather_api_cloud", current.get("cloud")),
            DataPoint("weather_api_feelslike_c", current.get("feelslike_c")),
            DataPoint("weather_api_vis_km", current.get("vis_km")),
            DataPoint("weather_api_uv", current.get("uv")),
            DataPoint("weather_api_gust_mpg", current.get("gust_mph"))
        ]
